/*-------------------------------------------------------------------------
 *
 * pipeline_query_fn.h
 *	 prototypes for functions in catalog/pipeline_query.c
 *
 * Copyright (c) 2013-2015, PipelineDB
 *
 * src/include/catalog/pipeline_query_fn.h
 *
 *-------------------------------------------------------------------------
 */
#ifndef PIPELINE_QUERY_FN_H
#define PIPELINE_QUERY_FN_H

#include "access/htup.h"
#include "datatype/timestamp.h"
#include "nodes/bitmapset.h"
#include "nodes/parsenodes.h"
#include "nodes/primnodes.h"
#include "storage/lock.h"
#include "utils/relcache.h"

typedef enum ContQueryType
{
	CONT_VIEW,
	CONT_TRANSFORM
} ContQueryType;

typedef struct ContQuery
{
	Oid id;
	RangeVar *name;
	Oid oid; /* OID in pipeline_query table */
	bool active;
	ContQueryType type;

	/* meta */
	Oid relid;
	char *sql;
	Oid matrelid;
	Oid osrelid;
	Query *cvdef;

	/* for view */
	RangeVar *matrel;
	Oid seqrelid;
	Oid pkidxid;
	Oid lookupidxid;
	int sw_step_factor;
	int sw_step_ms;
	uint64 sw_interval_ms;
	bool is_sw;
	AttrNumber ttl_attno;
	AttrNumber sw_attno;
	int ttl;

	/* for transform */
	Oid tgfn;
	int tgnargs;
	char **tgargs;
} ContQuery;

typedef Relation PipelineDDLLock;

#define AcquirePipelineDDLLock() heap_open(PipelineQueryRelationOid, AccessExclusiveLock);
#define ReleasePipelineDDLLock(lock) heap_close(lock, NoLock);

extern HeapTuple GetPipelineQueryTuple(RangeVar *name);

extern Oid DefineContinuousView(Oid relid, Query *query, Oid streamrelid, Oid matrel, Oid seqrel, int ttl, AttrNumber ttl_attno, Oid *pq_id);
extern void UpdateContViewRelIds(Oid cvid, Oid cvrelid, Oid osrelid, List *options);
extern void UpdateContViewIndexIds(Oid cvid, Oid pkindid, Oid lookupindid);
extern Oid DefineContinuousTransform(Oid relid, Query *query, Oid streamrelid, Oid typoid, Oid osrelid, List *options, Oid *ptgfnid);

extern Relation OpenCVRelFromMatRel(Relation matrel, LOCKMODE lockmode);
extern bool IsAContinuousView(RangeVar *name);
extern bool IsPipelineObject(RangeVar *name);
extern RangeVar *GetSWContinuousViewRangeVar(List *nodes);
extern bool IsAMatRel(RangeVar *name, RangeVar **cvname);
extern bool RelIdIsForMatRel(Oid relid, Oid *id);
extern bool RelIdIsForContView(Oid relid);
extern bool IsSWContView(RangeVar *name);
extern bool IsTTLContView(RangeVar *name);
extern RangeVar *GetMatRelName(RangeVar *cv);
extern RangeVar *GetCVNameFromMatRelName(RangeVar *matrel);

extern Bitmapset *GetContinuousQueryIds(void);
extern Bitmapset *GetContinuousViewIds(void);
extern Bitmapset *GetContinuousTransformIds(void);

extern Oid GetContQueryId(RangeVar *name);

extern ContQuery *GetContQueryForId(Oid id);
extern ContQuery *GetContQueryForView(RangeVar *cv_name);
extern ContQuery *GetContQueryForViewId(Oid id);
extern ContQuery *GetContQueryForTransformId(Oid id);

extern bool ContQuerySetActive(Oid id, bool active);

extern void GetTTLInfo(RangeVar *cvname, char **ttl_col, int *ttl);

extern void StorePipelineQueryReloptions(Oid relid, List *options);

#endif
